/** \file Datum.h
    \brief Declaration of Datum class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_Datum_h
#define profit_Datum_h

#include <map>
#include <set>
#include <string>

namespace profit {

  class DataView;

  /** \class Datum
      \brief Class encapsulating the concept of a named piece of information, which may or may
      not contain other Datum objects. In addition, a Datum may have one or more views.
  */
  class Datum {
    public:
      typedef std::set<DataView *> view_cont_type;

      typedef std::map<std::string, Datum> child_cont_type;
      typedef child_cont_type::value_type value_type;
      typedef child_cont_type::iterator iterator;
      typedef child_cont_type::const_iterator const_iterator;

      /** \brief Create a Datum object with the given name and value.
          \param name The name of the object.
          \param value The value of the object.
      */
      Datum(const std::string & name, const std::string & value = std::string());

      /// \brief Return the name of this object.
      std::string getName() const;

      /// \brief Return the value of this object.
      std::string getValue() const;

      /** \brief Set the value of this object.
          \param value The new value.
      */
      void setValue(const std::string & value);

      /** \brief Insert a child with the given properties. If a child with the same name already exists,
          exception is thrown.
          \param name The name of the child.
          \param value The value of the child.
      */
      Datum & insertChild(const std::string & name, const std::string & value = std::string());

      /// \brief Return an iterator pointing to the first child.
      iterator begin();

      /// \brief Return an iterator pointing to the one past the last child.
      iterator end();

      /// \brief Return a const_iterator pointing to the first child.
      const_iterator begin() const;

      /// \brief Return a const_iterator pointing to the one past the last child.
      const_iterator end() const;

      /// \brief Tell all views of this datum to update themselves.
      void updateAllViews();

      /** \brief Add a view to the container of known views which need to be notified when
          this object is assigned to.
          \param view The view being added.
      */
      void addView(DataView & view);

      /** \brief Add to the given container all views which are affected by this Datum.
          \param view_cont The output container of affected views.
      */
      void getAffectedViews(view_cont_type & view_cont) const;

    private:
      std::string m_name;
      std::string m_value;
      child_cont_type m_child;
      view_cont_type m_view;
  };

}

#endif
